from textwrap import dedent
import unittest
from lib2to3.refactor import RefactoringTool, get_fixers_from_package


class FixerTestBase(unittest.TestCase):
    def setUp(self):
        fixes = get_fixers_from_package('trollius_fixers')
        self.tool = RefactoringTool(fixes)

    def do(self, input, output):
        input = dedent(input)
        output = dedent(output)
        refactored = str(self.tool.refactor_string(input, 'dummy.py'))
        self.assertEqual(output, refactored)


class TestImports(FixerTestBase):
    def test_import(self):
        input = """
            import trollius
            trollius.get_event_loop()
            """
        output = """
            import asyncio
            asyncio.get_event_loop()
            """
        self.do(input, output)

    def test_from_import(self):
        input = """
            from trollius import (ensure_future, wait)
            from trollius import get_event_loop
            get_event_loop()
            """
        output = """
            from asyncio import (ensure_future, wait)
            from asyncio import get_event_loop
            get_event_loop()
            """
        self.do(input, output)

    def test_decorator(self):
        input = """
            import trollius
            @trollius.coroutine
            def coro():
                pass
        """
        output = """
            import asyncio
            @asyncio.coroutine
            def coro():
                pass
        """
        self.do(input, output)


class TestYieldFrom(FixerTestBase):
    def test_simple(self):
        input = """
            from trollius import From
            def coro():
                yield From(func())
            """
        output = """

            def coro():
                yield from(func())
            """
        self.do(input, output)

    def test_full_name(self):
        input = """
            import trollius
            def coro():
                yield trollius.From(func())
            """
        output = """
            import asyncio
            def coro():
                yield from(func())
            """
        self.do(input, output)

    def test_name_inside(self):
        input = """
            import trollius
            from trollius import From
            def coro(future):
                yield From(trollius.wait(future))
        """
        output = """
            import asyncio

            def coro(future):
                yield from(asyncio.wait(future))
        """
        self.do(input, output)


class TestRaiseReturn(FixerTestBase):
    def test_simple(self):
        input = """
            from trollius import Return
            def coro():
                raise Return(0)
        """
        output = """

            def coro():
                return (0)
        """
        self.do(input, output)

    def test_tuple(self):
        input = """
            from trollius import Return
            def coro():
                raise Return((1, 2))
        """
        output = """

            def coro():
                return (1, 2)
        """
        self.do(input, output)

    def test_full_name(self):
        input = """
            import trollius
            def coro():
                raise trollius.Return(1)
            """
        output = """
            import asyncio
            def coro():
                return (1)
            """
        self.do(input, output)

    def test_name_inside(self):
        input = """
            import trollius
            from trollius import From, Return
            def coro(future):
                raise Return((yield From(trollius.wait(future))))
        """
        output = """
            import asyncio

            def coro(future):
                return (yield from(asyncio.wait(future)))
        """
        self.do(input, output)


class TestImportFromReturn(FixerTestBase):
    def test_star(self):
        input = """
            from trollius import *
        """
        output = """
            from asyncio import *
        """
        self.do(input, output)

    def test_single(self):
        input = """
            from trollius import From
            from trollius import Return
            from trollius import ensure_future
        """
        output = """


            from asyncio import ensure_future
        """
        self.do(input, output)

    def test_multiple(self):
        input = """
            from trollius import From, Return
            from trollius import ensure_future, From, Return
            from trollius import From, Return, wait
            from trollius import (
                wait_for, From, ensure_future, Return, wait)
        """
        output = """

            from asyncio import ensure_future
            from asyncio import wait
            from asyncio import (
                wait_for, ensure_future, wait)
        """
        self.do(input, output)



from lib2to3.fixer_base import BaseFix
from lib2to3.fixer_util import Name, is_tuple, parenthesize, syms
from lib2to3 import pytree


class FixRaiseReturn(BaseFix):
    #BM_compatible = True
    PATTERN = """
        raise_stmt< 'raise' power=power< ('Return' | 'trollius' trailer< '.' 'Return' >)
            trailer=trailer< '(' expr=any ')' > > >
    """

    def transform(self, node, results):
        expr = results['expr']
        if is_tuple(expr):
            replacement = expr
        else:
            replacement = results['trailer']
        replacement.prefix = results['power'].prefix
        replacement.remove()
        return pytree.Node(syms.return_stmt, [Name('return', node.prefix), replacement])



from lib2to3.fixer_base import BaseFix
from lib2to3.fixer_util import Name, syms
from lib2to3 import pytree


class FixYieldFrom(BaseFix):
    BM_compatible = True
    PATTERN = """
        yield_expr< 'yield'
            power< name=('From' | 'trollius' trailer< '.' 'From' >)
                   expr=trailer< '(' any ')' > > >
    """

    def transform(self, node, results):
        name_nodes = results['name']
        name_nodes[0].replace(Name('from', prefix=name_nodes[0].prefix))
        for node in name_nodes[1:]:
            node.remove()



"""Replace @trollius.coroutine with @asyncio.coroutine.

This ought to be handled by fix_imports, but the lib2to3 fixer doesn't
handle decorators.
"""

from lib2to3.fixer_base import BaseFix
from lib2to3.fixer_util import Name


class FixDecorator(BaseFix):
    BM_compatible = True
    PATTERN = "decorator< '@' dotted_name< name='trollius' '.' any+ > any* >"

    def transform(self, node, results):
        name = results['name']
        name.replace(Name('asyncio', prefix=name.prefix))



import lib2to3.fixes.fix_imports


# We have to specify more than one module here, otherwise the code
# in lib2to3 gets a Leaf instead of a list. We use asyncio to pad out the
# list.
MAPPING = {'trollius': 'asyncio', 'asyncio': 'asyncio'}


class FixImports(lib2to3.fixes.fix_imports.FixImports):
    mapping = MAPPING



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='trollius-fixers',
    version='0.1',
    packages=find_packages(),
    scripts=['scripts/trollius2asyncio'],
    author='Bruce Merry',
    description='Convert trollius code to asyncio',
    long_description='''
        A tool based on lib2to3 for converting code using trollius to use
        asyncio. After installation, run *trollius2asyncio*. It works in the
        same way as 2to3.
        ''',
    license='PSF',
    keywords='2to3 asyncio trollius',
    url='https://github.com/bmerry/trollius_fixers',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Python Software Foundation License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: System :: Networking'
    ]
)



from lib2to3.fixer_base import BaseFix
from lib2to3.fixer_util import BlankLine, syms, token
from lib2to3 import pytree


class FixImportFromReturn(BaseFix):
    BM_compatible = True
    PATTERN = "import_from< 'from' 'trollius' 'import' ['('] imports=any [')'] >"

    def transform(self, node, results):
        imports = results['imports']
        if imports.type == syms.import_as_name or not imports.children:
            children = [imports]
        else:
            children = list(imports.children)
        for i in range(len(children) - 1, -1, -2):
            child = children[i]
            if child.type == token.NAME:
                name = child.value
            elif child.type == token.STAR:
                return
            else:
                assert child.type == syms.import_as_name
                name = child.children[0].value
            if name in ['From', 'Return']:
                if i + 1 < len(children):
                    children[i + 1].remove()
                    children[i].remove()
                    del children[i : i + 2]
                elif i > 0:
                    children[i].remove()
                    children[i - 1].remove()
                    del children[i - 1 : i + 1]
                else:
                    children[i].remove()
                    del children[i]
        if not children:
            new = BlankLine()
            new.prefix = node.prefix
            return new
        else:
            return None
